/*PROTECTED REGION ID(TucsenCam::main.cpp) ENABLED START*/
//=============================================================================
//
// file :        TucsenCam.cpp
//
// description : C++ source for the TucsenCam device server main.
//               The main rule is to initialise (and create) the Tango
//               system and to create the DServerClass singleton.
//                The main should be the same for every Tango device server.
//

#include <rnm_shm.h>
#include <rnm.h>
#include <k_rnmshare.h>
#include <rnmshare.h>

#include <tango.h>
#include <sched.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <sys/ioctl.h>

short parent = getpid();

int main(int argc,char *argv[])
{

	/*
	struct sched_param mysched;
	mysched.sched_priority = sched_get_priority_max(SCHED_RR) - 1;
	if( sched_setscheduler( 0, SCHED_RR, &mysched ) == -1 ) {
		cerr << "Launch server as root to gain maximum priority!" << endl;
 	}
	*/


#ifdef TUCSENCAM_RNM
	cout << "Initializing rnmshare in main.cpp to avoid sem bug" << endl;
	if (rnmshare_init(0) < 0) {
		cout << "Error initializing rnmshare (rnmshare_init) in main.cpp" << endl;
	}
	else {	
		int32_t bunch_number;
		char check; 
		rnm_bunch_number_get_value(0, 1, &bunch_number, &check);
		cout << "Init rnmshare done in main.cpp, bunchnumber: " << bunch_number << endl;
	}
#endif

	Tango::Util *tg;
	try
	{
		// Initialise the device server
		//----------------------------------------
		tg = Tango::Util::init(argc,argv);

		// Create the device server singleton 
		//	which will create everything
		//----------------------------------------
		tg->server_init(false);

		// Run the endless loop
		//----------------------------------------
		cout << "Ready to accept request" << endl;
		tg->server_run();
	}
	catch (bad_alloc)
	{
		cout << "Can't allocate memory to store device object !!!" << endl;
		cout << "Exiting" << endl;
	}
	catch (CORBA::Exception &e)
	{
		Tango::Except::print_exception(e);
		
		cout << "Received a CORBA_Exception" << endl;
		cout << "Exiting" << endl;
	}
	tg->server_cleanup();
	return(0);
}


/*PROTECTED REGION END*/	//	TucsenCam::main.cpp
